from setuptools import find_packages, setup

setup(
    name="pipd",
    packages=find_packages(exclude=[]),
    version="0.1.6",
    description="Utility functions for python data pipelines.",
    long_description_content_type="text/markdown",
    author="ElevenLabs",
    url="https://github.com/elevenlabs/pipd",
    keywords=["data processing", "pipeline"],
    install_requires=[],
    classifiers=[],
)
